@@ -89,6 +89,7 @@ Exception* knh_Exception_new__init(Ctx *ctx, Exception *o, String *e, String *ms
 {
 	knh_expt_t eid = EXPT_Exception;
 	if(IS_NOTNULL(e)) {
+		KNH_RCSETv(ctx, DP(o)->bag, e);
 		eid = knh_texpt_forname(ctx, knh_String_tobytes(e), EXPT_newid /*EXPT_unknown*/);
 	}
 	if(eid == EXPT_unknown) {
@@ -100,9 +101,9 @@ Exception* knh_Exception_new__init(Ctx *ctx, Exception *o, String *e, String *ms
 	}
 
 	DP(o)->flag = knh_tExpt_flag(DP(o)->eid);
-	KNH_SETv(ctx, DP(o)->bag, bag);
 	{
 		if(IS_NOTNULL(msg)) {
+			KNH_RCSETv(ctx, DP(o)->bag, msg);
 			knh_wbuf_t wbuf = knh_Context_wbuf(ctx);
 			knh_write__s(ctx, wbuf.w, EXPTN(DP(o)->eid));
 			knh_write__s(ctx, wbuf.w, "!!: ");
@@ -125,6 +126,7 @@ Exception* knh_Exception_new__init(Ctx *ctx, Exception *o, String *e, String *ms
 			KNH_SETv(ctx, DP(o)->message, new_String__wbuf(ctx, wbuf));
 		}
 	}
+	KNH_SETv(ctx, DP(o)->bag, bag);
 	return o;
 }
 
