@@ -68,7 +68,7 @@ knh_InputStream_struct_init(Ctx *ctx, knh_InputStream_struct *b, int init, Objec
 /* ------------------------------------------------------------------------ */
 
 void
-knh_InputStream_struct_traverse(Ctx *ctx, knh_InputStream_struct *b, f_traverse ftr)
+knh_InputStream_struct_traverse(Ctx *ctx, knh_InputStream_struct *b, knh_ftraverse ftr)
 {
 	if(IS_SWEEP(ftr) && b->fd != -1) {
 		b->driver.fclose(ctx, b->fd);
@@ -93,7 +93,7 @@ Object *knh_InputStream_open(Ctx *ctx, InputStream *o, String *urn)
 	else {
 		DP(o)->driver = *(knh_System_getIODriver(ctx, knh_bytes_first(fname, loc)));
 	}
-	DP(o)->fd = DP(o)->driver.fopen(ctx, UP(o), fname, "r");
+	DP(o)->fd = DP(o)->driver.fopen(ctx, o, NULL, fname, "r");
 	if(DP(o)->fd != -1) {
 		DP(o)->bufsiz = DP(o)->driver.bufsiz;
 		if(DP(o)->bufsiz > 0) {
@@ -107,7 +107,7 @@ Object *knh_InputStream_open(Ctx *ctx, InputStream *o, String *urn)
 	else {
 		DP(o)->driver = knh_System_getDefaultIODriver();
 		char buff[FILENAME_BUFSIZ];
-		knh_snprintf(buff, sizeof(buff), "IO!!: %s", (char*)fname.buf);
+		knh_snprintf(buff, sizeof(buff), "IO!!: '%s'", (char*)fname.buf);
 		return (Object*)new_Nue__s(ctx, buff);
 	}
 	return (Object*)o;
@@ -251,7 +251,7 @@ KNHAPI(InputStream*) new_FileInputStream(Ctx *ctx, knh_bytes_t file)
 		(InputStream*)new_Object_malloc(ctx, FLAG_InputStream, CLASS_InputStream, sizeof(knh_InputStream_struct));
 	knh_InputStream_struct_init(ctx, DP(o), 0, NULL);
 	DP(o)->driver = *(knh_System_getIODriver(ctx, STEXT("file")));
-	DP(o)->fd = DP(o)->driver.fopen(ctx, UP(o), file, "r");
+	DP(o)->fd = DP(o)->driver.fopen(ctx, o, NULL, file, "r");
 	if(DP(o)->fd != -1) {
 		DP(o)->bufsiz = DP(o)->driver.bufsiz;
 		if(DP(o)->bufsiz > 0) {
@@ -288,6 +288,14 @@ KNHAPI(InputStream*) new_BytesInputStream(Ctx *ctx, Bytes *ba, size_t s, size_t
 	return o;
 }
 
+/* ------------------------------------------------------------------------ */
+/* @map Bytes InputStream! */
+
+InputStream* knh_Bytes_InputStream(Ctx *ctx, Bytes *o, Mapper *mpr)
+{
+	return new_BytesInputStream(ctx, o, 0, knh_Bytes_size(o));
+}
+
 /* ======================================================================== */
 /* [Bytes] */
 
