@@ -62,11 +62,12 @@ knh_Compiler_struct_init(Ctx *ctx, knh_Compiler_struct *b, int init, Object *cs)
 	KNH_INITv(b->method,   KNH_NULL);
 
 	b->nastep = 0;
-	KNH_INITv(b->elf, new_Bytes(ctx, 1024));
+	KNH_INITv(b->elf, new_Bytes(ctx, 4096 * 8));  TODO();
 	KNH_INITv(b->dwarf, new_Bytes(ctx, 1024));
 	KNH_INITv(b->labelIdDictIdx, new_DictIdx(ctx, 256, 1));
 	KNH_INITv(b->labelAddrDictSet, new_DictSet(ctx, 256));
 	KNH_INITv(b->lstacks, new_Array(ctx, CLASS_String, 8));
+	KNH_INITv(b->finallyStmt, KNH_NULL);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -79,24 +80,25 @@ knh_Compiler_struct_init(Ctx *ctx, knh_Compiler_struct *b, int init, Object *cs)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_Compiler_struct_traverse(Ctx *ctx, knh_Compiler_struct *b, f_traverse gc)
+void knh_Compiler_struct_traverse(Ctx *ctx, knh_Compiler_struct *b, knh_ftraverse ftr)
 {
 	knh_int_t i;
 	for(i = 0; i < KONOHA_LOCALVAR_SIZE; i++) {
-		gc(ctx, b->vars[i].value);
+		ftr(ctx, b->vars[i].value);
 	}
 
-	if(IS_SWEEP(gc)) {
-		KNH_FREE(b->vars, KONOHA_LOCALVAR_SIZE * sizeof(knh_cfield_t));
+	if(IS_SWEEP(ftr)) {
+		KNH_FREE(ctx, b->vars, KONOHA_LOCALVAR_SIZE * sizeof(knh_cfield_t));
 	}
 
-	gc(ctx, UP(b->ns));
-	gc(ctx, UP(b->method));
-	gc(ctx, UP(b->elf));
-	gc(ctx, UP(b->dwarf));
-	gc(ctx, UP(b->labelIdDictIdx));
-	gc(ctx, UP(b->labelAddrDictSet));
-	gc(ctx, UP(b->lstacks));
+	ftr(ctx, UP(b->ns));
+	ftr(ctx, UP(b->method));
+	ftr(ctx, UP(b->elf));
+	ftr(ctx, UP(b->dwarf));
+	ftr(ctx, UP(b->labelIdDictIdx));
+	ftr(ctx, UP(b->labelAddrDictSet));
+	ftr(ctx, UP(b->lstacks));
+	ftr(ctx, UP(b->finallyStmt));
 }
 
 /* ======================================================================== */
@@ -387,6 +389,7 @@ void knh_Compiler_startCompilation(Ctx *ctx, Compiler *cpr, Stmt *stmt)
 	knh_Compiler_setCancelled(cpr, 0);
 
 	KNH_ASM_SETLINE(ctx, cpr, SP(stmt)->line);
+	KNH_ASM_INIT(ctx, cpr);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -421,6 +424,7 @@ void knh_Compiler_endCompilation(Ctx *ctx, Compiler *cpr)
 	KNH_ASSERT(IS_Method(mtd));
 
 	if(!knh_Compiler_isCancelled(cpr)) {
+		KNH_ASM_HALT(ctx, cpr);
 		knh_Compiler_remapAddress(ctx, cpr);
 	}
 
@@ -438,7 +442,6 @@ void knh_Compiler_endCompilation(Ctx *ctx, Compiler *cpr)
 	else {
 		KLRCode *vmc = new_KLRCode(ctx, DP(cpr)->fileid, knh_Bytes_tobytes(DP(cpr)->elf), knh_Bytes_tobytes(DP(cpr)->dwarf));
 		knh_Method_setKLRCode(ctx, mtd, vmc);
-		DBG2_DUMP(ctx, mtd, KNH_NULL, "Compiled Code");
 		if(knh_Compiler_isStopped(cpr)) {
 			if(DP(mtd)->mn != METHODN_lambda) {
 				char bufcm[CLASSNAME_BUFSIZ];
@@ -447,6 +450,12 @@ void knh_Compiler_endCompilation(Ctx *ctx, Compiler *cpr)
 			}
 			knh_Method_toAbstract(ctx, mtd);
 		}
+		else {
+			knh_sfp_t *sfp = (ctx)->ebp + 2;
+			KNH_SETv(ctx, sfp[-1].o, mtd);
+			knh_KLRCode_exec(ctx, sfp); /* code threading */
+			DBG2_DUMP(ctx, mtd, KNH_NULL, "Compiled Code");
+		}
 	}
 	knh_Compiler_clear(ctx, cpr);
 }
