@@ -109,9 +109,9 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in, int join)
 		switch(ch) {
 		case '\n':
 			knh_Token_add_whitespace(ctx, tkctx, tbuf, in, join);
-
 		goto INDENT_PART;
 
+		//case 1:
 		case '\t': case ' ': case '\v': case '\r':
 			knh_Token_add_whitespace(ctx, tkctx, tbuf, in, join);
 			prev = ch;
@@ -948,6 +948,47 @@ void knh_Token_padd(Ctx *ctx, knh_tokenctx_t *tkctx, Token *tk)
 
 /* ------------------------------------------------------------------------ */
 
+void knh_Token_addTypeParam(Ctx *ctx, Token *tkt, Array *a, size_t s)
+{
+	size_t i;
+	for(i = s; i < knh_Array_size(a); i++) {
+		Token *tk = (Token*)knh_Array_n(a, i);
+		if(SP(tk)->tt == TT_GT) goto L_PASS2;
+	}
+	return ;
+
+	L_PASS2:;
+	knh_wbuf_t wbuf = knh_Context_wbuf(ctx);
+	knh_write(ctx, wbuf.w, knh_Token_tobytes(tkt));
+	knh_putc(ctx, wbuf.w, '<');
+	knh_Array_remove(ctx, a, s);
+	i = s;
+	while(1) {
+		Token *tk = (Token*)knh_Array_n(a, i);
+		if(SP(tk)->tt == TT_GT) {
+			if(DP(tk)->tt_next == TT_NOT) {
+				tk = (Token*)knh_Array_n(a, i+1);
+				DP(tkt)->tt_next = DP(tk)->tt_next;
+				knh_Token_setNotNullType(tkt, 1);
+				knh_Array_remove(ctx, a, i);
+				knh_Array_remove(ctx, a, i);
+			}
+			else {
+				DP(tkt)->tt_next = DP(tk)->tt_next;
+				knh_Array_remove(ctx, a, i);
+			}
+			break;
+		}
+		knh_write(ctx, wbuf.w, knh_Token_tobytes(tk));
+		knh_Array_remove(ctx, a, i);
+	}
+	knh_putc(ctx, wbuf.w, '>');
+	KNH_SETv(ctx, DP(tkt)->data, new_String__wbuf(ctx, wbuf));
+
+}
+
+/* ------------------------------------------------------------------------ */
+
 void knh_Token_retokens(Ctx *ctx, Token *o)
 {
 	Array *a = (Array*)DP(o)->data;
@@ -957,27 +998,42 @@ void knh_Token_retokens(Ctx *ctx, Token *o)
 			Token *tk = (Token*)knh_Array_n(a, i);
 			if(SP(tk)->tt == TT_TYPEN) {
 				Token *tkn = (Token*)knh_Array_n(a, i+1);
+				if(SP(tkn)->tt == TT_LT) {
+					knh_Token_addTypeParam(ctx, tk, a, i+1);
+				}
 				if(SP(tkn)->tt == TT_BRANCET && IS_NULL(DP(tkn)->data)) {
-					DEBUG3("%s[]", knh_Token_tochar(tk));
 					knh_Token_setArrayType(tk, 1);
-					knh_Token_setNotNullType(tk, 1);
 					DP(tk)->tt_next = DP(tkn)->tt_next;
-					knh_Array_remove(ctx, a, i+1);
+					if(DP(tkn)->tt_next == TT_NOT) {
+						knh_Token_setNotNullType(tk, 1);
+						DP(tk)->tt_next = DP((Token*)knh_Array_n(a, i+2))->tt_next;
+						knh_Array_remove(ctx, a, i+1);
+						knh_Array_remove(ctx, a, i+1);
+					}
+					else {
+						knh_Array_remove(ctx, a, i+1);
+					}
 				}
 			}
 			if(SP(tk)->tt == TT_NAME && ISB(knh_Token_tobytes(tk), "byte")) {
 				Token *tkn = (Token*)knh_Array_n(a, i+1);
 				if(SP(tkn)->tt == TT_BRANCET && IS_NULL(DP(tkn)->data)) {
-					DEBUG3("%s[]", knh_Token_tochar(tk));
 					SP(tk)->tt = TT_TYPEN;
 					KNH_SETv(ctx, DP(tk)->data, knh_tClass[CLASS_Bytes].sname);
-					knh_Token_setNotNullType(tk, 1);
 					if(i>0) {
 						Token *tkp = (Token*)knh_Array_n(a, i-1);
 						DP(tkp)->tt_next = TT_TYPEN;
 					}
 					DP(tk)->tt_next = DP(tkn)->tt_next;
-					knh_Array_remove(ctx, a, i+1);
+					if(DP(tkn)->tt_next == TT_NOT) {
+						knh_Token_setNotNullType(tk, 1);
+						DP(tk)->tt_next = DP((Token*)knh_Array_n(a, i+2))->tt_next;
+						knh_Array_remove(ctx, a, i+1);
+						knh_Array_remove(ctx, a, i+1);
+					}
+					else {
+						knh_Array_remove(ctx, a, i+1);
+					}
 				}
 			}
 		}
@@ -1022,6 +1078,11 @@ void knh_Token_add_space(Ctx *ctx, knh_tokenctx_t *tkctx, knh_wbuf_t tbuf, Input
 					knh_wbuf_clear(tbuf);
 					return;
 				}
+				if(t.buf[0] == '?' && t.buf[2] == '\0') {
+					knh_Token_setPluralType(tkp, 1);
+					knh_wbuf_clear(tbuf);
+					return;
+				}
 			}
 		}
 	}
