@@ -38,11 +38,7 @@
 #define KONOHA_VERSION   PACKAGE_VERSION
 #else
 #define KONOHA_NAME     "konoha"
-#define KONOHA_VERSION  "0.2.4"
-#endif
-
-#ifndef KNH_DBGMODE2
-#define KNH_FASTMODE    1
+#define KONOHA_VERSION  "0.2.5-rc2"
 #endif
 
 #define KONOHA_URL		"http://konoha.sourceforge.jp/"
@@ -51,6 +47,12 @@
 #define KONOHA_BUILDID_TOOOLD  360
 #define KNH_CHECK_VERSION(id)  ((id) <= KONOHA_BUILDID_TOOOLD)
 
+/* ======================================================================== */
+/* [OPTIONS] */
+
+#define KNH_USING_RCGC    1                /* Reference Counting */
+#define KNH_USING_DRCGC   1
+
 /* ======================================================================== */
 
 #ifdef KONOHA_OS__LINUX
@@ -78,7 +80,7 @@
 /* cpp -dM /dev/null */
 
 #ifdef __GNUC__
-	#define KONOHA_CC_VERSION __VERSION__
+	#define KONOHA_CC_VERSION "GCC " __VERSION__
 	#ifdef __i386__
 		#define KNH_CC_FASTCALL   __attribute__((fastcall))
 	#else
@@ -107,6 +109,12 @@
 			#define KONOHA_OS__TB       1
 		#endif
 	#endif/*KONOHA_OS*/
+
+/* Linux kernel style macros */
+
+//#define likely(x)	__builtin_expect((x), 1)
+//#define unlikely(x)	__builtin_expect((x), 0)
+
 #endif/*__GNUC__*/
 
 #ifdef _MSC_VER
@@ -132,10 +140,19 @@
 #define KONOHA_CC_VERSION "UNKNOWN COMPILER"
 #endif
 
-/* ======================================================================== */
-/* [DEFUALT] */
 
-#define KONOHA_WITH_RCGC  1                /* Reference Counting */
+#ifndef likely
+#define likely(x)	   x
+#define unlikely(x)	   x
+#endif
+
+/* ======================================================================== */
+/* [OPTIONS_DEPS] */
+
+#ifdef KNH_USING_DRCGC
+#define KNH_USING_RCGC 1
+#endif
+
 
 #ifdef KONOHA_CC_WITH_ECLIPSE
 	#define KONOHA_MONOLITHIC  1
