@@ -17,25 +17,29 @@
 #ifdef KONOHA_OS__LINUX
 #define		KNH_USING_POSIX 1
 #define		KNH_USING_UNIX  1
+#define     KNH_USING_SOCKET 1
 #define		KONOHA_OS_ENCODING "UTF-8"
 #ifdef HAVE_ICONV_H
 #define		KNH_USING_ICONV 1
 #endif
-#ifdef HAVE_REGEX_H
+//#ifdef HAVE_REGEX_H
 #define		KNH_USING_REGEX 1
-#endif
+//#endif
 #define		KONOHA_OS_LINEFEED "\n"
 #define		KONOHA_OS_DLLEXT   ".so"
 #ifdef KONOHA_CC_WITH_ECLIPSE
 #define 	KNH_USING_READLINE 1
+#define		KONOHA_MONOLITHIC  1
 #endif
 #endif
 
 #ifdef KONOHA_OS__MACOSX
 #define 	KNH_USING_POSIX    1
 #define 	KNH_USING_UNIX     1
+#define 	KNH_USING_SOCKET   1
 #define 	KNH_USING_ICONV    1
 #define 	KNH_USING_REGEX    1
+#define 	KNH_USING_SQLITE3  1
 #define 	KONOHA_OS_ENCODING "UTF-8"
 #define 	KONOHA_OS_LINEFEED "\n"
 #define 	KONOHA_OS_DLLEXT ".dylib"
@@ -63,6 +67,7 @@
 #define 	KNH_EXPORTS(T)     __declspec(dllexport) T __cdecl
 
 #define 	KONOHA_OS_DLLEXT ".dll"
+#define		KONOHA_OS_FILE_SEPARATOR '\\'
 #endif
 
 #ifdef KONOHA_OS__MINGW
@@ -75,8 +80,8 @@
 #define 	KNHAPI(T) __declspec(dllexport) T __cdecl
 #endif
 #define 	KNH_EXPORTS(T)  __declspec(dllexport) T __cdecl
-#define KONOHA_OS_DLLEXT ".dll"
-
+#define 	KONOHA_OS_DLLEXT ".dll"
+#define		KONOHA_OS_FILE_SEPARATOR '\\'
 #ifndef HAVE_CONFIG_H
 #define 	KNH_USING_ICONV       1
 #define 	HAVE_LOCALCHARSET_H   1
@@ -106,6 +111,14 @@
 /* ======================================================================== */
 /* [KNHAPI] */
 
+#ifndef KNHAPI_
+#ifdef KNHAPI
+#define KNHAPI_(T)       KNHAPI(T)
+#else
+#define KNHAPI_(T)       T KNH_CC_FASTCALL
+#endif
+#endif
+
 #ifndef KNHAPI
 #define KNHAPI(T)         T
 #define KNH_EXPORTS(T)    T
@@ -117,6 +130,17 @@
 #define KNH_EXPORTS(T)   T
 #endif
 
+#ifndef KONOHA_OS_FILE_SEPARATOR
+#define KONOHA_OS_FILE_SEPARATOR  '/'
+#endif
+
+#ifdef KNH_CC_LABELPTR
+#define KNH_USING_THREADEDCODE 1
+#else
+#undef KNH_USING_THREADEDCODE 1
+#endif
+
+
 /* ======================================================================== */
 /* [CONFIG_H] */
 
