@@ -65,6 +65,10 @@ int knh_Method_pctoline(Method *mtd, knhvmc_t *pc);
 //#define VM_FILE                   ""
 //#define VM_LINE                   0
 
+/* ======================================================================== */
+/* [BOXING/UNBOXING] */
+
+
 /* ======================================================================== */
 
 #define KNH_RETURN(ctx, sfp, v)      {\
@@ -98,6 +102,15 @@ int knh_Method_pctoline(Method *mtd, knhvmc_t *pc);
 
 /* ======================================================================== */
 
+#define VM_MOV__ngc(ctx, v1, v2)   \
+	knh_Object_RCinc(v2); \
+	knh_Object_RCdec((Object*)v1); \
+	v1 = v2; \
+
+#ifdef KNH_USING_DRCGC
+#define VM_MOV(ctx, v1, v2)  VM_MOV__ngc(ctx, v1, v2)
+
+#else/*KNH_USING_DRCGC*/
 #define VM_MOV(ctx, v1, v2)   \
 	knh_Object_RCinc(v2); \
 	knh_Object_RCdec(v1); \
@@ -106,10 +119,8 @@ int knh_Method_pctoline(Method *mtd, knhvmc_t *pc);
 	} \
 	v1 = v2; \
 
-#define VM_MOV__ngc(ctx, v1, v2)   \
-	knh_Object_RCinc(v2); \
-	knh_Object_RCdec((Object*)v1); \
-	v1 = v2; \
+#endif/*KNH_USING_DRCGC*/
+
 
 /* ======================================================================== */
 
@@ -118,29 +129,82 @@ int knh_Method_pctoline(Method *mtd, knhvmc_t *pc);
 
 #define VM_RET(ctx, v)      \
 	VM_MOV__ngc(ctx, sfp[-1].o, v); \
-	return; \
+	return ; \
 
 #define VM_RET__OIDX(ctx, v, idx)      \
 	VM_MOV__ngc(ctx, sfp[-1].o, oidx_(v, idx)); \
-	return; \
+	return ; \
 
-#define VM_MOV_(ctx,v,v2)            KNH_SETv(ctx,v,v2)
+//#define VM_MOV_(ctx,v,v2)            KNH_SETv(ctx,v,v2)
 
 #define VM_MOVE(ctx,v,v2)            VM_MOV(ctx,v,v2)
 #define VM_MOVS(ctx,v,v2)            VM_MOV(ctx,v,v2)
 #define VM_MOVO(ctx,v,v2)            VM_MOV(ctx,v,v2)
 #define VM_MOVI(ctx,o,idx,v2)        VM_MOV(ctx,oidx_(o,idx),v2)
 
-#define VM_MOVE__OIDX(ctx,v,o,idx)   VM_MOV(ctx,v,oidx_(o,idx))
-#define VM_MOVS__OIDX(ctx,v,o,idx)   VM_MOV(ctx,v,oidx_(o,idx))
-#define VM_MOVO__OIDX(ctx,v,o,idx)   VM_MOV(ctx,v,oidx_(o,idx))
-#define VM_MOVI__OIDX(ctx,o,idx,o2,idx2)   VM_MOV(ctx,oidx_(o,idx),oidx_(o2,idx2))
+#define VM_MOVE__OIDX(ctx,v,o,idx)          VM_MOV(ctx,v,oidx_(o,idx))
+#define VM_MOVS__OIDX(ctx,v,o,idx)          VM_MOV(ctx,v,oidx_(o,idx))
+#define VM_MOVO__OIDX(ctx,v,o,idx)          VM_MOV(ctx,v,oidx_(o,idx))
+#define VM_MOVI__OIDX(ctx,o,idx,o2,idx2)    VM_MOV(ctx,oidx_(o,idx),oidx_(o2,idx2))
 
 #define VM_MOVSFP_IFNUL(ctx, n, v2) \
 	if(IS_NULL(sfp[n].o)) {\
 		KNH_SETv(ctx, sfp[n].o, v2);\
 	}\
 
+
+/* ======================================================================== */
+
+#define VM_BBOXS(ctx, n) { \
+		KNH_SETv(ctx, sfp[n].o, sfp[n].bvalue == 0 ? KNH_FALSE : KNH_TRUE);\
+	}\
+
+#define VM_BUNBOXS(ctx, n) { \
+		sfp[n].bvalue = IS_TRUE(sfp[n].o);\
+	}\
+
+#define VM_BBOXE(ctx, n) { \
+		KNH_SETv(ctx, ebp[n].o, ebp[n].bvalue == 0 ? KNH_FALSE : KNH_TRUE);\
+	}\
+
+#define VM_BUNBOXE(ctx, n) { \
+		ebp[n].bvalue = IS_TRUE(ebp[n].o);\
+	}\
+
+#define VM_IBOXS(ctx, n) { \
+		KNH_SETv(ctx, sfp[n].o, new_Int(ctx, sfp[n].ivalue));\
+	}\
+
+#define VM_IUNBOXS(ctx, n) { \
+		sfp[n].ivalue = (sfp[n].i)->value;\
+	}\
+
+#define VM_IBOXE(ctx, n) { \
+		KNH_SETv(ctx, ebp[n].o, new_Int(ctx, ebp[n].ivalue));\
+	}\
+
+#define VM_IUNBOXE(ctx, n) { \
+		ebp[n].ivalue = (ebp[n].i)->value;\
+	}\
+
+#define VM_FBOXS(ctx, n) { \
+		KNH_SETv(ctx, sfp[n].o, new_Int(ctx, sfp[n].fvalue));\
+	}\
+
+#define VM_FUNBOXS(ctx, n) { \
+		sfp[n].fvalue = (sfp[n].f)->value;\
+	}\
+
+#define VM_FBOXE(ctx, n) { \
+		KNH_SETv(ctx, ebp[n].o, new_Int(ctx, ebp[n].fvalue));\
+	}\
+
+#define VM_FUNBOXE(ctx, n) { \
+		ebp[n].fvalue = (ebp[n].f)->value;\
+	}\
+
+/* ======================================================================== */
+
 #define _VM_PUT_(ctx,v) \
 	VM_MOV(ctx, ((Context*)ctx)->ebp[0].o, v); \
 
@@ -189,8 +253,17 @@ int knh_Method_pctoline(Method *mtd, knhvmc_t *pc);
 		((Context*)ctx)->ebp = &sfp[n]; \
 	} \
 
+/* ------------------------------------------------------------------------ */
+
+#define KNH_CHECKSTACK(ctx) \
+	if(unlikely((ctx)->stacksize - ((ctx)->ebp - (ctx)->stack) < 256)) {\
+		KNH_THROWs(ctx, "StackOverflow!!"); \
+	}\
+
 #define KNH_SCALL(ctx, n) { \
+		KNH_CHECKSTACK(ctx); \
 		knh_sfp_t *mbp_ = ctx->ebp - (n) ; \
+		KNH_CHECKSTACK(ctx); \
 		KNH_ASSERT(IS_Method(mbp_[0].mtd)); \
 		mbp_[0].op = 0; \
 		(mbp_[0].mtd)->fcall_1(ctx, mbp_ + 1); \
@@ -199,6 +272,7 @@ int knh_Method_pctoline(Method *mtd, knhvmc_t *pc);
 	} \
 
 #define VM_SCALL(ctx, n) { \
+		KNH_CHECKSTACK(ctx); \
 		knh_sfp_t *mbp_ = ((Context*)ctx)->ebp - (n) ; \
 		KNH_ASSERT(IS_Method(mbp_[0].mtd)); \
 		KNH_ASSERT(IS_NOTNULL(mbp_[1].o) || knh_Method_isNullBase(mbp_[0].mtd)); \
@@ -209,8 +283,9 @@ int knh_Method_pctoline(Method *mtd, knhvmc_t *pc);
 	} \
 
 #define VM_NSCALL(ctx, n) { \
+		KNH_CHECKSTACK(ctx); \
 		knh_sfp_t *mbp_ = ((Context*)ctx)->ebp - (n) ; \
-		if(IS_NULL(mbp_[1].o)) { \
+		if(unlikely(IS_NULL(mbp_[1].o))) { \
 			knh_throwException(ctx, new_Exception__Nue(ctx, (Nue*)(mbp_[1].o)), _HERE_); \
 		} \
 		else { \
@@ -223,6 +298,7 @@ int knh_Method_pctoline(Method *mtd, knhvmc_t *pc);
 	} \
 
 #define VM_CALL(ctx, n, mn) { \
+		KNH_CHECKSTACK(ctx); \
 		knh_sfp_t *mbp_ = ctx->ebp - (n) ; \
 		KNH_SETv(ctx, mbp_[0].o, knh_Method_ufind(ctx, knh_Object_cid(mbp_[1].o), mn)); \
 		KNH_ASSERT(IS_Method(mbp_[0].mtd)); \
@@ -233,6 +309,7 @@ int knh_Method_pctoline(Method *mtd, knhvmc_t *pc);
 	} \
 
 #define VM_DCALL(ctx, n, mn) { \
+		KNH_CHECKSTACK(ctx); \
 		knh_sfp_t *mbp_ = ((Context*)ctx)->ebp - (n) ; \
 		KNH_SETv(ctx, mbp_[0].o, knh_Method_ufind(ctx, knh_Object_cid(mbp_[1].o), mn)); \
 		KNH_ASSERT(IS_Method(mbp_[0].mtd)); \
@@ -259,7 +336,7 @@ int knh_Method_pctoline(Method *mtd, knhvmc_t *pc);
 		knh_Mapper_t *mpr_ = (knh_Mapper_t*)mpr; \
 		VM_DEBUG_ASSERT(IS_Mapper(mpr)); \
 		KNH_SETv(ctx, ebp_(0), (mpr_)->fmap_1(ctx, ebp_(0), mpr_)); \
-		if(IS_NULL(ebp_(0))) { \
+		if(unlikely(IS_NULL(ebp_(0)))) { \
 			knh_throwException(ctx, new_Exception__Nue(ctx, (Nue*)ebp_(0)), _HERE_); \
 		} \
 	} \
@@ -289,28 +366,28 @@ int knh_Method_pctoline(Method *mtd, knhvmc_t *pc);
 			knh_Mapper_t *mpr_ = knh_tMapper_find(ctx, knh_Object_cid(ebp_(0)), tcid); \
 			KNH_SETv(ctx, ebp_(0), (mpr_)->fmap_1(ctx, ebp_(0), mpr_)); \
 		} \
-		if(IS_NULL(ebp_(0))) { \
+		if(unlikely(IS_NULL(ebp_(0)))) { \
 			knh_throwException(ctx, new_Exception__Nue(ctx, (Nue*)ebp_(0)), _HERE_); \
 		} \
 	} \
 
 #define VM_NULLCHK(ctx, v) \
-	if(IS_NULL(v)) { \
+	if(unlikely(IS_NULL(v))) { \
 		knh_throwException(ctx, new_Exception__Nue(ctx, (Nue*)v), _HERE_); \
 	} \
 
 #define VM_NULLCHK__OIDX(ctx, v, idx) \
-	if(IS_NULL(oidx_(v,idx))) { \
+	if(unlikely(IS_NULL(oidx_(v,idx)))) { \
 		knh_throwException(ctx, new_Exception__Nue(ctx, (Nue*)(oidx_(v,idx))), _HERE_); \
 	} \
 
 #define VM_TYPECHK(ctx, type, v) \
-	if(!knh_Object_opTypeOf(ctx, v, type)) { \
+	if(unlikely(!knh_Object_opTypeOf(ctx, v, type))) { \
 		knh_throwException(ctx, new_Exception__type(ctx, v, type), _HERE_); \
 	} \
 
 #define VM_TYPECHK__OIDX(ctx, type, v, idx) \
-	if(!knh_Object_opTypeOf(ctx, oidx_(v, idx), type)) { \
+	if(unlikely(!knh_Object_opTypeOf(ctx, oidx_(v, idx), type))) { \
 		knh_throwException(ctx, new_Exception__type(ctx, oidx_(v, idx), type), _HERE_); \
 	} \
 
@@ -531,6 +608,36 @@ int knh_Method_pctoline(Method *mtd, knhvmc_t *pc);
 
 #define VM_STACKDUMP(ctx)   knh_sfp_dump(ctx, sfp)
 
+/* ------------------------------------------------------------------------ */
+
+#define VM_INIT(ctx) { \
+		KLRCode *c_ = (KLRCode*)DP(sfp[-1].mtd)->code; \
+		knhvmc_codethreading(ctx, pc, globalMachineToJump); \
+		DP(c_)->code = pc + KNH_ASMC_INIT_SIZ; \
+		DP(c_)->size = DP(c_)->size - KNH_ASMC_INIT_SIZ; \
+		return; \
+	}\
+
+/* ------------------------------------------------------------------------ */
+
+#define VM_IADD(ctx) { \
+		((Context*)ctx)->ebp--; \
+		Object *n_ = UP(new_Int(ctx, ((Int*)(ebp_(0)))->value + ((Int*)(ebp_(1)))->value)); \
+		VM_MOV(ctx, ((Context*)ctx)->ebp[0].o, n_); \
+	}\
+
+#define VM_ISUB(ctx) { \
+		((Context*)ctx)->ebp--; \
+		Object *n_ = UP(new_Int(ctx, ((Int*)(ebp_(0)))->value - ((Int*)(ebp_(1)))->value)); \
+		VM_MOV(ctx, ((Context*)ctx)->ebp[0].o, n_); \
+	}\
+
+#define VM_ILT(ctx) { \
+		((Context*)ctx)->ebp--; \
+		Object *n_ = (((Int*)(ebp_(0)))->value < ((Int*)(ebp_(1)))->value) ? KNH_TRUE : KNH_FALSE ; \
+		VM_MOV(ctx, ((Context*)ctx)->ebp[0].o, n_); \
+	}\
+
 #define VM_NOP(ctx)
 
 /* ------------------------------------------------------------------------ */
